import { P2P as P2PTypes } from '@shardeum-foundation/lib-types';
import * as Account from '../dbstore/accounts';
import * as Transaction from '../dbstore/transactions';
import * as Receipt from '../dbstore/receipts';
import * as OriginalTxsData from '../dbstore/originalTxsData';
import * as State from '../State';
import { DataType, GossipData } from './GossipData';
import * as OriginalTxDB from '../dbstore/originalTxsData';
export declare let storingAccountData: boolean;
export interface ReceiptVerificationResult {
    success: boolean;
    failedReasons?: string[];
    nestedCounterMessages?: string[];
}
/**
 * Validate type and field existence of the receipt data before processing it further
 * @param receipt
 * @returns boolean
 */
export declare const validateReceiptType: (receipt: Receipt.Receipt | Receipt.ArchiverReceipt) => boolean;
/**
 * Verifies the receipt data to ensure its integrity and validity.
 *
 * @param receipt - The receipt object which can be either a `Receipt.Receipt` or `Receipt.ArchiverReceipt`.
 * @returns A promise that resolves to an object containing a `success` boolean indicating the verification result.
 *
 * The function performs the following checks:
 * 1. Validates the transaction ID by comparing the generated transaction ID with the incoming transaction ID.
 * 2. Logs the time taken between the receipt timestamp and the current time if verbose logging is enabled.
 * 3. Checks if the receipt cycle is older than 2 cycles and logs an error if true.
 * 4. Retrieves the cycle shard data and logs an error if not found.
 * 5. If the receipt is a global modification receipt, validates it using AJV and returns the result.
 * 6. If the receipt is a non-global transaction receipt, validates it and returns the result.
 *
 * The function logs appropriate errors and counts events using `nestedCountersInstance` for various failure scenarios.
 */
export declare const verifyReceiptData: (receipt: Receipt.Receipt | Receipt.ArchiverReceipt) => Promise<{
    success: boolean;
}>;
/**
 * Verifies the given archiver receipt.
 *
 * @param {Receipt.ArchiverReceipt | Receipt.Receipt} receipt - The receipt to verify. It can be either an ArchiverReceipt or a generic Receipt.
 * @returns {Promise<ReceiptVerificationResult>} A promise that resolves to a ReceiptVerificationResult object indicating the success or failure of the verification process.
 *
 * The function performs the following verifications:
 * 1. If `config.verifyAppReceiptData` is enabled, it verifies the application receipt data.
 *    - If the verification fails, it returns a failure result with the reasons.
 *    - If the receipt is valid but does not need to be saved, it returns a failure result with the reasons.
 * 2. If `config.verifyAccountData` is enabled, it verifies the account data.
 *    - If the verification fails, it returns a failure result with the reasons.
 *
 * The function returns a success result if all enabled verifications pass.
 */
export declare const verifyArchiverReceipt: (receipt: Receipt.ArchiverReceipt | Receipt.Receipt) => Promise<ReceiptVerificationResult>;
/**
 * Stores receipt data in the database.
 *
 * @param {Receipt.Receipt[] | Receipt.ArchiverReceipt[]} receipts - Array of receipts to be stored.
 * @param {string} [senderInfo=''] - Optional sender information.
 * @param {boolean} [verifyData=false] - Flag to indicate if the receipt data should be verified.
 * @param {boolean} [saveOnlyGossipData=false] - Flag to indicate if only gossip data should be saved.
 * @returns {Promise<void>} - A promise that resolves when the operation is complete.
 *
 * @remarks
 * This function processes and stores receipt data. It performs validation and verification of receipts,
 * updates account and transaction data, and handles bulk insertion of data into the database.
 *
 * The function skips processing if the receipts array is empty or invalid. It also skips receipts that
 * have already been processed or are in the validation map.
 *
 * If `verifyData` is true, the function verifies the receipt data and handles any verification failures.
 *
 * The function processes each receipt, updating account and transaction data, and performs bulk insertion
 * when the number of receipts, accounts, transactions, or processed transactions reaches a specified bucket size.
 *
 * If the archiver is not active and the processed receipts map exceeds 2000 entries, the map is cleared.
 */
export declare const storeReceiptData: (receipts: Receipt.Receipt[] | Receipt.ArchiverReceipt[], senderInfo?: string, verifyData?: boolean, saveOnlyGossipData?: boolean) => Promise<void>;
export declare const validateCycleData: (cycleRecord: P2PTypes.CycleCreatorTypes.CycleData) => boolean;
export declare const storeCycleData: (cycles?: P2PTypes.CycleCreatorTypes.CycleData[]) => Promise<void>;
interface StoreAccountParam {
    accounts?: Account.AccountsCopy[];
    receipts?: Transaction.Transaction[];
}
export declare const storeAccountData: (restoreData?: StoreAccountParam) => Promise<void>;
export declare const storeOriginalTxData: (originalTxsData?: OriginalTxsData.OriginalTxData[], senderInfo?: string, saveOnlyGossipData?: boolean) => Promise<void>;
interface validateResponse {
    success: boolean;
    reason?: string;
    error?: string;
}
export declare const validateOriginalTxDataSchema: (originalTxData: OriginalTxsData.OriginalTxData) => boolean;
export declare const validateGossipData: (data: GossipData) => validateResponse;
export declare const processGossipData: (gossipdata: GossipData) => void;
export declare const collectMissingTxDataFromArchivers: () => Promise<void>;
export declare const collectMissingReceipts: (senders: string[], txId: string, txTimestamp: number) => Promise<void>;
type QueryTxDataFromArchiversResponse = Receipt.Receipt[] | OriginalTxDB.OriginalTxData[] | null;
export declare const queryTxDataFromArchivers: (archiver: State.ArchiverNodeInfo, txDataType: DataType, txIdList: [
    string,
    number
][]) => Promise<QueryTxDataFromArchiversResponse>;
export declare function cleanOldReceiptsMap(timestamp: number): void;
export declare function cleanOldOriginalTxsMap(timestamp: number): void;
export declare const scheduleMissingTxsDataQuery: () => void;
export {};
