"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.totalNodeCount = exports.activeNodeCount = exports.applyNodeListChange = exports.parse = exports.parseRecord = exports.ChangeSquasher = exports.reversed = exports.NodeStatus = void 0;
const NodeList = require("../NodeList");
var NodeStatus;
(function (NodeStatus) {
    NodeStatus["ACTIVE"] = "active";
    NodeStatus["SYNCING"] = "syncing";
    NodeStatus["REMOVED"] = "removed";
})(NodeStatus = exports.NodeStatus || (exports.NodeStatus = {}));
function reversed(thing) {
    const arr = Array.isArray(thing) ? thing : Array.from(thing);
    let i = arr.length - 1;
    const reverseIterator = {
        next: () => {
            const done = i < 0;
            // eslint-disable-next-line security/detect-object-injection
            const value = done ? undefined : arr[i];
            i--;
            return { value, done };
        },
    };
    return {
        [Symbol.iterator]: () => reverseIterator,
    };
}
exports.reversed = reversed;
class ChangeSquasher {
    constructor() {
        this.final = {
            added: [],
            removed: [],
            updated: [],
        };
        this.addedIds = new Set();
        this.removedIds = new Set();
        this.seenUpdates = new Map();
    }
    addChange(change) {
        for (const id of change.removed) {
            // Ignore if id is already removed
            if (this.removedIds.has(id))
                continue;
            // Mark this id as removed
            this.removedIds.add(id);
        }
        for (const update of change.updated) {
            // Ignore if update.id is already removed
            if (this.removedIds.has(update.id))
                continue;
            // Skip if it's already seen in the update
            if (this.seenUpdates.has(update.id))
                continue;
            // Mark this id as updated
            this.seenUpdates.set(update.id, update);
            // console.log('seenUpdates', this.seenUpdates, update)
        }
        for (const joinedConsensor of reversed(change.added)) {
            // Ignore if it's already been added
            if (this.addedIds.has(joinedConsensor.id))
                continue;
            // Ignore if joinedConsensor.id is already removed
            if (this.removedIds.has(joinedConsensor.id)) {
                continue;
            }
            // Check if this id has updates
            const update = this.seenUpdates.get(joinedConsensor.id);
            if (update) {
                // If so, put them into final.updated
                this.final.updated.unshift(update);
                this.seenUpdates.delete(joinedConsensor.id);
            }
            // Add joinedConsensor to final.added
            this.final.added.unshift(joinedConsensor);
            // Mark this id as added
            this.addedIds.add(joinedConsensor.id);
        }
    }
}
exports.ChangeSquasher = ChangeSquasher;
function parseRecord(record) {
    // For all nodes described by activated, make an update to change their status to active
    const activated = record.activated.map((id) => ({
        id,
        activeTimestamp: record.start,
        status: NodeStatus.ACTIVE,
    }));
    const refreshAdded = [];
    const refreshUpdated = [];
    for (const refreshed of record.refreshedConsensors) {
        // const node = NodeList.nodes.get(refreshed.id)
        const node = NodeList.getNodeInfoById(refreshed.id);
        if (node) {
            // If it's in our node list, we update its counterRefreshed
            // (IMPORTANT: update counterRefreshed only if its greater than ours)
            if (record.counter > node.counterRefreshed) {
                refreshUpdated.push({
                    id: refreshed.id,
                    counterRefreshed: record.counter,
                });
            }
        }
        else {
            // If it's not in our node list, we add it...
            refreshAdded.push(NodeList.fromP2PTypesNode(refreshed));
            // and immediately update its status to ACTIVE
            // (IMPORTANT: update counterRefreshed to the records counter)
            refreshUpdated.push({
                id: refreshed.id,
                status: NodeStatus.ACTIVE,
                counterRefreshed: record.counter,
            });
        }
    }
    // Logger.mainLogger.debug('parseRecord', record.counter, {
    //   added: [...record.joinedConsensors],
    //   removed: [...record.apoptosized],
    //   updated: [...activated, ...refreshUpdated],
    // })
    const added = (record.joinedConsensors || []).map((joinedConsensor) => NodeList.fromP2PTypesJoinedConsensor(joinedConsensor));
    return {
        added: [...added],
        removed: [...record.apoptosized, ...record.removed, ...record.appRemoved],
        updated: [...activated, ...refreshUpdated],
    };
}
exports.parseRecord = parseRecord;
function parse(record) {
    const changes = parseRecord(record);
    // const mergedChange = deepmerge.all<Change>(changes)
    // return mergedChange
    return changes;
}
exports.parse = parse;
function applyNodeListChange(change) {
    // console.log('change', change)
    if (change.added.length > 0) {
        const nodesBycycleJoined = {};
        for (const node of change.added) {
            const joinedConsensor = node;
            const consensorInfo = {
                ip: joinedConsensor.externalIp,
                port: joinedConsensor.externalPort,
                publicKey: joinedConsensor.publicKey,
                id: joinedConsensor.id,
            };
            if (!nodesBycycleJoined[joinedConsensor.cycleJoined]) {
                nodesBycycleJoined[joinedConsensor.cycleJoined] = [consensorInfo];
            }
            else
                nodesBycycleJoined[joinedConsensor.cycleJoined].push(consensorInfo);
        }
        for (const cycleJoined in nodesBycycleJoined) {
            // eslint-disable-next-line security/detect-object-injection
            NodeList.addNodes(NodeList.NodeStatus.SYNCING, nodesBycycleJoined[cycleJoined]);
        }
    }
    // This is not needed though since no removed nodes are ever added to this list
    // If we ever add removed nodes to this list, we need to update to removeNodes by publicKey instead of id
    // Commenting out for now
    // if (change.removed.length > 0) {
    //   NodeList.removeNodes(change.removed)
    // }
    if (change.updated.length > 0) {
        const activatedPublicKeys = change.updated.reduce((keys, update) => {
            const nodeInfo = NodeList.getNodeInfoById(update.id);
            if (nodeInfo) {
                keys.push(nodeInfo.publicKey);
            }
            return keys;
        }, []);
        NodeList.setStatus(NodeList.NodeStatus.ACTIVE, activatedPublicKeys);
    }
}
exports.applyNodeListChange = applyNodeListChange;
function activeNodeCount(cycle) {
    return (cycle.active +
        cycle.activated.length +
        -cycle.apoptosized.length +
        -cycle.removed.length +
        -cycle.appRemoved.length +
        -cycle.lost.length);
}
exports.activeNodeCount = activeNodeCount;
function totalNodeCount(cycle) {
    return (cycle.syncing +
        cycle.joinedConsensors.length +
        cycle.active +
        //    cycle.activated.length -      // don't count activated because it was already counted in syncing
        -cycle.apoptosized.length +
        -cycle.removed.length +
        -cycle.appRemoved.length
    // -cycle.lost.length
    );
}
exports.totalNodeCount = totalNodeCount;
//# sourceMappingURL=CycleParser.js.map