"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleLostArchivers = void 0;
const CycleDB = require("./dbstore/cycles");
const Cycles = require("./Data/Cycles");
const Logger = require("./Logger");
const NodeList = require("./NodeList");
const Config_1 = require("./Config");
const Data_1 = require("./Data/Data");
const P2P_1 = require("./P2P");
const Crypto_1 = require("./Crypto");
const allowedArchiversManager_1 = require("./shardeum/allowedArchiversManager");
let shouldSendRefutes = false;
/**
 * Checks for the existence of our own public key in either the
 * 'refutedArchivers', 'lostArchivers', or 'removedArchivers' fields of the
 * supplied record.
 * If found in 'refutedArchivers', we'll stop sending refutes.
 * If found in 'lostArchivers', we'll schedule a refute in the next cycle's Q1.
 * If found in 'removedArchivers', we'll shut down.
 */
function handleLostArchivers(record) {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const debug = (message, ...args) => Logger.mainLogger.debug(message, ...args);
    debug('>> handleLostArchivers()');
    debug('  config.ARCHIVER_PUBLIC_KEY: ' + Config_1.config.ARCHIVER_PUBLIC_KEY);
    // debug('  record: ' + StringUtils.safeStringify(record, null, 2))
    if (record && record.refutedArchivers && record.lostArchivers && record.removedArchivers) {
        if (record.refutedArchivers.some((publicKey) => publicKey === Config_1.config.ARCHIVER_PUBLIC_KEY)) {
            // if self is in 'refutedArchivers' field, stop sending refutes
            debug('archiver was found in `refutedArchivers` and will stop sending refutes');
            shouldSendRefutes = false;
        }
        else if (record.lostArchivers.some((publicKey) => publicKey === Config_1.config.ARCHIVER_PUBLIC_KEY)) {
            // if self is in 'lostArchivers' field, schedule a refute in the next cycle's Q1
            debug("archiver was found in `lostArchivers` and will send a refute in the next cycle's Q1");
            shouldSendRefutes = true;
            scheduleRefute();
        }
        else if (record.removedArchivers.some((publicKey) => publicKey === Config_1.config.ARCHIVER_PUBLIC_KEY)) {
            // if self is in 'removedArchivers' field, shut down
            debug('archiver was found in `removedArchivers`, shutting down');
            die();
        }
    }
    debug('<< handleLostArchivers()');
}
exports.handleLostArchivers = handleLostArchivers;
/**
 * Schedules to send a refute during the next cycle's Q1.
 */
async function scheduleRefute() {
    if (!shouldSendRefutes) {
        console.log('skipping refute scheduling');
        return;
    }
    console.log('scheduling refute');
    const latestCycleInfo = await CycleDB.queryLatestCycleRecords(1);
    const latestCycle = latestCycleInfo[0];
    const { quarterDuration, startQ1 } = (0, Data_1.calcIncomingTimes)(latestCycle);
    // ms until q1. add 500ms to make sure we're in q1
    const delay = startQ1 + 4 * quarterDuration - Date.now() + 500;
    console.log(delay);
    setTimeout(sendRefute, delay);
}
/**
 * Sends a refute to 5 random active nodes.
 */
async function sendRefute() {
    if (!shouldSendRefutes) {
        console.log('skipping refute sending');
        return;
    }
    console.log('sending refute');
    const refuteMsg = (0, Crypto_1.sign)({
        archiver: Config_1.config.ARCHIVER_PUBLIC_KEY,
        cycle: Cycles.getCurrentCycleMarker(),
    });
    const nodes = NodeList.getRandomActiveNodes(5);
    for (const node of nodes) {
        try {
            await (0, P2P_1.postJson)(`http://${node.ip}:${node.port}/lost-archiver-refute`, refuteMsg);
        }
        catch (e) {
            Logger.mainLogger.warn(`Failed to send refute to ${node.ip}:${node.port}:`, e);
            scheduleRefute();
        }
    }
}
/**
 * Shuts down the archiver with exit code 2.
 */
function die() {
    Logger.mainLogger.debug('Archiver was found in `removedArchivers` and will exit now without sending a leave request');
    allowedArchiversManager_1.allowedArchiversManager.stopWatching();
    process.exit(2);
}
//# sourceMappingURL=LostArchivers.js.map