"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isBucketVerified = exports.getCheckpointSyncRange = exports.getOldestPendingOrFailedCheckpointStatus = exports.getCheckpointStatusesByUnifiedStatus = exports.getCheckpointStatus = exports.bulkUpdateCheckpointStatusField = exports.updateCheckpointStatusField = exports.upsertCheckpointStatus = exports.fieldMapping = exports.CheckpointStatusType = void 0;
const db = require("./sqlite3storage");
const _1 = require(".");
const Logger = require("../Logger");
const Config_1 = require("../Config");
var CheckpointStatusType;
(function (CheckpointStatusType) {
    CheckpointStatusType["CYCLE"] = "cycle";
    CheckpointStatusType["RECEIPT"] = "receipt";
    CheckpointStatusType["ORIGINAL_TX"] = "original_tx";
})(CheckpointStatusType = exports.CheckpointStatusType || (exports.CheckpointStatusType = {}));
// Map enum values directly to field names
exports.fieldMapping = {
    [CheckpointStatusType.CYCLE]: 'cycleStatus',
    [CheckpointStatusType.RECEIPT]: 'receiptStatus',
    [CheckpointStatusType.ORIGINAL_TX]: 'originalTxStatus',
};
/**
 * Inserts or updates a checkpoint status record
 * @param status The checkpoint status to insert or update
 */
async function upsertCheckpointStatus(status) {
    try {
        const sql = `
      INSERT OR REPLACE INTO checkpoint_status 
      (cycle, unifiedStatus, cycleStatus, receiptStatus, originalTxStatus, created_at) 
      VALUES (?, ?, ?, ?, ?, ?)
    `;
        // Calculate unifiedStatus based on the other three statuses
        const unifiedStatus = status.cycleStatus && status.receiptStatus && status.originalTxStatus;
        await db.run(_1.checkpointStatusDatabase, sql, [
            status.cycle,
            unifiedStatus,
            status.cycleStatus,
            status.receiptStatus,
            status.originalTxStatus,
            status.created_at,
        ]);
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug(`Successfully upserted checkpoint status for cycle ${status.cycle}, unifiedStatus ${status.unifiedStatus}`);
        }
    }
    catch (err) {
        Logger.mainLogger.error('Error upserting checkpoint status:', err);
        throw err;
    }
}
exports.upsertCheckpointStatus = upsertCheckpointStatus;
/**
 * Updates a specific status field for a checkpoint
 * @param cycle The cycle number
 * @param statusField The status field to update ('cycleStatus', 'receiptStatus', or 'originalTxStatus')
 * @param value The boolean value to set
 */
async function updateCheckpointStatusField(cycle, statusField, value) {
    try {
        // Create a new status object with default values
        const newStatus = {
            cycle,
            unifiedStatus: false,
            cycleStatus: false,
            receiptStatus: false,
            originalTxStatus: false,
            created_at: Date.now(),
        };
        // Get the current status (if it exists)
        const currentStatus = await getCheckpointStatus(cycle);
        // If current status exists, copy all its values
        if (currentStatus) {
            newStatus.cycleStatus = currentStatus.cycleStatus;
            newStatus.receiptStatus = currentStatus.receiptStatus;
            newStatus.originalTxStatus = currentStatus.originalTxStatus;
        }
        // Get the field name to update based on the status field type
        const fieldToUpdate = exports.fieldMapping[statusField];
        newStatus[fieldToUpdate] = value;
        // Calculate the unified status
        if (currentStatus) {
            let sql = `UPDATE checkpoint_status SET ${fieldToUpdate} = ? WHERE cycle = ?`;
            await db.run(_1.checkpointStatusDatabase, sql, [value, newStatus.cycle]);
            const status = await getCheckpointStatus(newStatus.cycle);
            sql = `UPDATE checkpoint_status SET unifiedStatus = ? WHERE cycle = ?`;
            await db.run(_1.checkpointStatusDatabase, sql, [
                status?.cycleStatus && status?.receiptStatus && status?.originalTxStatus,
                newStatus.cycle,
            ]);
        }
        else {
            const sql = `
      INSERT INTO checkpoint_status 
      (cycle, unifiedStatus, cycleStatus, receiptStatus, originalTxStatus, created_at) 
      VALUES (?, ?, ?, ?, ?, ?)
    `;
            await db.run(_1.checkpointStatusDatabase, sql, [
                newStatus.cycle,
                newStatus.unifiedStatus,
                newStatus.cycleStatus,
                newStatus.receiptStatus,
                newStatus.originalTxStatus,
                newStatus.created_at,
            ]);
        }
        if (Config_1.config.VERBOSE) {
            Logger.mainLogger.debug(`Updated checkpoint status for cycle ${cycle}, ${statusField} to ${value}`);
        }
    }
    catch (err) {
        Logger.mainLogger.error('Error updating checkpoint status field:', err);
        throw err;
    }
}
exports.updateCheckpointStatusField = updateCheckpointStatusField;
/**
 * Bulk updates a specific status field (e.g., ORIGINAL_TX) for all cycles in [startCycle, endCycle].
 * This approach:
 *   - Grabs existing rows in one SELECT (so we know other status fields).
 *   - Updates/creates rows in memory.
 *   - Uses one multi-row upsert to write them back.
 */
async function bulkUpdateCheckpointStatusField(statusField, value, startCycle, endCycle, cycles) {
    try {
        if (startCycle !== undefined && endCycle !== undefined && cycles !== undefined) {
            throw new Error('Only one of startCycle, endCycle, or cycles should be provided');
        }
        let existingRows = [];
        let existingMap = new Map();
        // Handle range-based updates
        if (startCycle !== undefined && endCycle !== undefined) {
            if (endCycle < startCycle) {
                throw new Error(`Invalid range: endCycle (${endCycle}) < startCycle (${startCycle})`);
            }
            existingRows = await getCheckpointStatusForRange(startCycle, endCycle);
            // Convert to a map for quick lookup by cycle number
            for (const row of existingRows) {
                existingMap.set(row.cycle, row);
            }
            // Create entries for missing cycles in the range
            for (let cycle = startCycle; cycle <= endCycle; cycle++) {
                if (!existingMap.has(cycle)) {
                    existingMap.set(cycle, {
                        cycle,
                        unifiedStatus: false,
                        cycleStatus: false,
                        receiptStatus: false,
                        originalTxStatus: false,
                        created_at: Date.now(),
                    });
                }
            }
        }
        // Handle specific cycles
        if (cycles && cycles.length > 0) {
            const cyclesStatus = new Map();
            await Promise.allSettled(cycles.map(async (cycle) => {
                const status = await getCheckpointStatus(cycle);
                if (status) {
                    cyclesStatus.set(cycle, status);
                }
                else {
                    cyclesStatus.set(cycle, {
                        cycle,
                        unifiedStatus: false,
                        cycleStatus: false,
                        receiptStatus: false,
                        originalTxStatus: false,
                        created_at: Date.now(),
                    });
                }
            }));
            existingMap = cyclesStatus;
        }
        // Build the new/updated list of statuses in memory
        const toUpsertList = [];
        for (let cycle of existingMap.keys()) {
            // Get the current status (should always exist at this point)
            let currentStatus = existingMap.get(cycle);
            // Update the desired field using the fieldMapping
            const fieldToUpdate = exports.fieldMapping[statusField];
            currentStatus[fieldToUpdate] = value;
            // Recompute the unifiedStatus
            currentStatus.unifiedStatus =
                currentStatus.cycleStatus && currentStatus.receiptStatus && currentStatus.originalTxStatus;
            currentStatus.created_at = Date.now();
            toUpsertList.push(currentStatus);
        }
        // Perform a bulk upsert in one shot
        if (toUpsertList.length > 0) {
            await bulkUpsertCheckpointStatus(toUpsertList);
        }
        if (Config_1.config.VERBOSE) {
            if (startCycle !== undefined && endCycle !== undefined) {
                Logger.mainLogger.debug(`Bulk updated field "${statusField}" to ${value} for cycles [${startCycle}..${endCycle}]`);
            }
            else if (cycles) {
                Logger.mainLogger.debug(`Bulk updated field "${statusField}" to ${value} for specific cycles: ${cycles.join(', ')}`);
            }
        }
    }
    catch (err) {
        Logger.mainLogger.error('Error in bulkUpdateCheckpointStatusField:', err);
        throw err;
    }
}
exports.bulkUpdateCheckpointStatusField = bulkUpdateCheckpointStatusField;
async function bulkUpsertCheckpointStatus(statusList) {
    if (statusList.length === 0)
        return;
    // We are constructing multiple placeholders for a single multi-row insert
    const placeholders = [];
    const values = [];
    for (const s of statusList) {
        placeholders.push('(?, ?, ?, ?, ?, ?)');
        values.push(s.cycle, s.unifiedStatus, s.cycleStatus, s.receiptStatus, s.originalTxStatus, s.created_at);
    }
    // Upsert by 'cycle' using ON CONFLICT DO UPDATE
    const sql = `
    INSERT INTO checkpoint_status (
      cycle,
      unifiedStatus,
      cycleStatus,
      receiptStatus,
      originalTxStatus,
      created_at
    )
    VALUES ${placeholders.join(', ')}
    ON CONFLICT(cycle) DO UPDATE SET
      unifiedStatus=excluded.unifiedStatus,
      cycleStatus=excluded.cycleStatus,
      receiptStatus=excluded.receiptStatus,
      originalTxStatus=excluded.originalTxStatus,
      created_at=excluded.created_at
  `;
    try {
        await db.run(_1.checkpointStatusDatabase, sql, values);
    }
    catch (err) {
        Logger.mainLogger.error('Error in bulkUpsertCheckpointStatus:', err);
        throw err;
    }
    if (Config_1.config.VERBOSE) {
        Logger.mainLogger.debug(`Bulk upserted ${statusList.length} checkpoint status rows.`);
    }
}
/**
 * Gets a checkpoint status by cycle
 * @param cycle The cycle number
 */
async function getCheckpointStatus(cycle) {
    try {
        const sql = `
      SELECT * FROM checkpoint_status
      WHERE cycle = ?
    `;
        const row = await db.get(_1.checkpointStatusDatabase, sql, [cycle]);
        if (!row) {
            return null;
        }
        // Add type assertion to fix TypeScript errors
        const typedRow = row;
        return {
            cycle: typedRow.cycle,
            unifiedStatus: Boolean(typedRow.unifiedStatus),
            cycleStatus: Boolean(typedRow.cycleStatus),
            receiptStatus: Boolean(typedRow.receiptStatus),
            originalTxStatus: Boolean(typedRow.originalTxStatus),
            created_at: typedRow.created_at,
        };
    }
    catch (error) {
        Logger.mainLogger.error(`Error getting checkpoint status: ${error}`);
        throw error;
    }
}
exports.getCheckpointStatus = getCheckpointStatus;
/**
 * Gets all checkpoint statuses for a range of cycles
 * @param startCycle The start cycle number
 * @param endCycle The end cycle number
 */
async function getCheckpointStatusForRange(startCycle, endCycle) {
    const sql = `
    SELECT *
    FROM checkpoint_status
    WHERE cycle BETWEEN ? AND ?
    ORDER BY cycle
  `;
    const rows = await db.all(_1.checkpointStatusDatabase, sql, [startCycle, endCycle]);
    return rows;
}
/**
 * Gets all checkpoint statuses with a specific unified status
 * @param unified Whether to get checkpoints with unified status true or false
 */
async function getCheckpointStatusesByUnifiedStatus(unified) {
    try {
        const sql = `
      SELECT * FROM checkpoint_status
      WHERE unifiedStatus = ?
    `;
        const rows = await db.all(_1.checkpointStatusDatabase, sql, [unified ? 1 : 0]);
        return rows.map((row) => {
            // Add type assertion to fix TypeScript errors
            const typedRow = row;
            return {
                cycle: typedRow.cycle,
                unifiedStatus: Boolean(typedRow.unifiedStatus),
                cycleStatus: Boolean(typedRow.cycleStatus),
                receiptStatus: Boolean(typedRow.receiptStatus),
                originalTxStatus: Boolean(typedRow.originalTxStatus),
                created_at: typedRow.created_at,
            };
        });
    }
    catch (error) {
        Logger.mainLogger.error(`Error getting checkpoint statuses by unified status: ${error}`);
        throw error;
    }
}
exports.getCheckpointStatusesByUnifiedStatus = getCheckpointStatusesByUnifiedStatus;
/**
 * Gets the oldest pending or failed checkpoint status
 * @returns The oldest pending or failed checkpoint status or null if none found
 */
async function getOldestPendingOrFailedCheckpointStatus() {
    try {
        const sql = `
      SELECT * FROM checkpoint_status
      WHERE unifiedStatus = ?
      ORDER BY cycle ASC
      LIMIT 1
    `;
        const result = await db.get(_1.checkpointStatusDatabase, sql, [false]);
        if (!result) {
            return null;
        }
        // Add type assertion to fix TypeScript errors
        const typedResult = result;
        return {
            cycle: typedResult.cycle,
            unifiedStatus: typedResult.unifiedStatus,
            cycleStatus: typedResult.cycleStatus,
            receiptStatus: typedResult.receiptStatus,
            originalTxStatus: typedResult.originalTxStatus,
            created_at: typedResult.created_at,
        };
    }
    catch (err) {
        Logger.mainLogger.error('Error getting oldest pending or failed checkpoint status:', err);
        throw err;
    }
}
exports.getOldestPendingOrFailedCheckpointStatus = getOldestPendingOrFailedCheckpointStatus;
/**
 * Gets the range of cycles that need syncing
 * @returns Object with min and max cycle numbers that need syncing
 */
async function getCheckpointSyncRange() {
    try {
        const sql = `
      SELECT MIN(cycle) as minCycle, MAX(cycle) as maxCycle
      FROM checkpoint_status
      WHERE unifiedStatus = ?
    `;
        const result = await db.get(_1.checkpointStatusDatabase, sql, [false]);
        // Add type assertion to fix TypeScript errors
        const typedResult = result;
        if (!typedResult || typedResult.minCycle === null || typedResult.maxCycle === null) {
            return null;
        }
        return {
            minCycle: typedResult.minCycle,
            maxCycle: typedResult.maxCycle,
        };
    }
    catch (err) {
        Logger.mainLogger.error('Error getting checkpoint sync range:', err);
        throw err;
    }
}
exports.getCheckpointSyncRange = getCheckpointSyncRange;
async function isBucketVerified(bucketID, endBucketID) {
    if (endBucketID !== undefined && endBucketID < bucketID) {
        return false; // Invalid range
    }
    if (endBucketID !== undefined) {
        // Check a range of buckets
        const sql = `
      SELECT cycle, unifiedStatus
      FROM checkpoint_status
      WHERE cycle >= ? AND cycle <= ?
    `;
        const results = await db.all(_1.checkpointStatusDatabase, sql, [bucketID, endBucketID]);
        // If no results or fewer results than expected, return false
        if (!results || results.length === 0 || results.length < endBucketID - bucketID + 1) {
            return false;
        }
        // Check if all buckets in the range have unifiedStatus = true/1
        return results.every((row) => row.unifiedStatus === true || row.unifiedStatus === 1);
    }
    else {
        // Check a single bucket
        const sql = `
      SELECT unifiedStatus
      FROM checkpoint_status
      WHERE cycle = ?
    `;
        const result = (await db.get(_1.checkpointStatusDatabase, sql, [bucketID]));
        if (!result) {
            return false;
        }
        return result.unifiedStatus === true || result.unifiedStatus === 1;
    }
}
exports.isBucketVerified = isBucketVerified;
//# sourceMappingURL=checkpointStatus.js.map