"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTxSenderAddress = exports.getTransactionObj = exports.validateTransferFromSecureAccount = exports.isInternalTx = exports.isInternalTXGlobal = exports.verifyTransaction = exports.syncKeysFromNetworkConfig = exports.scheduleMultiSigKeysSyncFromNetConfig = exports.getMultisigPublicKeys = void 0;
const Logger = require("../Logger");
const internalTxType_1 = require("../types/internalTxType");
const crypto = require("@shardeum-foundation/lib-crypto-utils");
const P2P = require("../P2P");
const NodeList = require("../NodeList");
const Utils_1 = require("../Utils");
const security_1 = require("../types/security");
const path_1 = require("path");
const fs_1 = require("fs");
const tx_1 = require("@ethereumjs/tx");
const util_1 = require("@ethereumjs/util");
const lib_net_1 = require("@shardeum-foundation/lib-net");
const lib_types_1 = require("@shardeum-foundation/lib-types");
const Config_1 = require("../Config");
let secureAccountsFilePath = (0, path_1.join)(__dirname, '..', '..', 'static', 'genesis-secure-accounts.json');
let secureAccountDataMap = null;
if (process.env.LOAD_JSON_GENESIS_SECURE_ACCOUNTS) {
    const GSAFilePath = process.env.LOAD_JSON_GENESIS_SECURE_ACCOUNTS;
    GSAFilePath.trim();
    try {
        if ((0, fs_1.existsSync)((0, path_1.join)(__dirname, '..', '..', 'static', GSAFilePath))) {
            secureAccountsFilePath = (0, path_1.join)(__dirname, '..', '..', 'static', GSAFilePath);
            console.log('secureAccounts: genesis secure path set to:', GSAFilePath);
        }
        else {
            throw new Error('secureAccounts: path to the following genesis secure accounts file is incorrect:' + GSAFilePath);
        }
    }
    catch (e) {
        throw new Error('secureAccounts: error setting genesis secure accounts file path: ' + e);
    }
}
const getSecureAccounts = () => {
    if (!secureAccountDataMap) {
        try {
            const data = (0, fs_1.readFileSync)(secureAccountsFilePath, 'utf8');
            const jsonData = JSON.parse(data);
            secureAccountDataMap = new Map(jsonData.map((account) => [account.Name, account]));
        }
        catch (error) {
            Logger.mainLogger.error(`Error reading secure accounts file: ${error}`);
            secureAccountDataMap = new Map();
        }
    }
    return secureAccountDataMap;
};
let multisigKeys = {};
let minMultiSigRequiredForGlobalTxs = 3;
const getMultisigPublicKeys = () => {
    return multisigKeys;
};
exports.getMultisigPublicKeys = getMultisigPublicKeys;
const scheduleMultiSigKeysSyncFromNetConfig = () => {
    setInterval(async () => {
        console.log('Executing syncKeysFromNetworkConfig on interval...');
        await (0, exports.syncKeysFromNetworkConfig)();
    }, Config_1.config.multisigKeysSyncFromNetworkInternal * 1000); // will sync for multi sig keys from network each 10 mins
};
exports.scheduleMultiSigKeysSyncFromNetConfig = scheduleMultiSigKeysSyncFromNetConfig;
const syncKeysFromNetworkConfig = async () => {
    try {
        const queryFn = async (node) => {
            const REQUEST_NETCONFIG_TIMEOUT_SECOND = 3;
            try {
                const response = await P2P.getJson(`http://${node.ip}:${node.port}/netconfig`, REQUEST_NETCONFIG_TIMEOUT_SECOND);
                return response;
            }
            catch (error) {
                Logger.mainLogger.error(`Error querying node ${node.ip}:${node.port}: ${error} while updating dev public keys and multisig keys from /netconfig`);
                return null;
            }
        };
        const equalityFn = (responseA, responseB) => {
            return (JSON.stringify(responseA?.config?.debug?.multisigKeys) ===
                JSON.stringify(responseB?.config?.debug?.multisigKeys) &&
                JSON.stringify(responseA?.config?.debug?.minMultiSigRequiredForGlobalTxs) ===
                    JSON.stringify(responseB?.config?.debug?.minMultiSigRequiredForGlobalTxs));
        };
        // Get the list of 10 max random active nodes or the first node if no active nodes are available
        const nodes = NodeList.getActiveNodeCount() > 0 ? NodeList.getRandomActiveNodes(10) : [NodeList.getFirstNode()];
        const tallyItem = await (0, Utils_1.robustQuery)(nodes, queryFn, equalityFn, 3 // Redundancy (minimum 3 nodes should return the same result to reach consensus)
        );
        if (tallyItem?.value?.config?.debug) {
            const newMultisigKeys = tallyItem.value.config.debug.multisigKeys;
            const newMinMultiSigRequired = tallyItem.value.config.debug.minMultiSigRequiredForGlobalTxs;
            if (newMultisigKeys &&
                typeof newMultisigKeys === 'object' &&
                JSON.stringify(newMultisigKeys) !== JSON.stringify(multisigKeys)) {
                multisigKeys = newMultisigKeys;
            }
            if (typeof newMinMultiSigRequired === 'number' &&
                newMinMultiSigRequired > 0 &&
                newMinMultiSigRequired !== minMultiSigRequiredForGlobalTxs) {
                minMultiSigRequiredForGlobalTxs = newMinMultiSigRequired;
            }
        }
    }
    catch (error) {
        Logger.mainLogger.error('error in syncKeysFromNetworkConfig', error);
    }
};
exports.syncKeysFromNetworkConfig = syncKeysFromNetworkConfig;
const verifyTransaction = (tx) => {
    try {
        if (isInternalTx(tx)) {
            if (isInternalTXGlobal(tx)) {
                return { result: 'pass', reason: 'valid' };
            }
            else if (tx.internalTXType === internalTxType_1.InternalTXType.ChangeConfig ||
                tx.internalTXType === internalTxType_1.InternalTXType.ChangeNetworkParam) {
                const multiSigPublicKeys = (0, exports.getMultisigPublicKeys)();
                const is_array_sig = Array.isArray(tx.sign) === true;
                const requiredSigs = Math.max(3, minMultiSigRequiredForGlobalTxs);
                // Ensure old single sig / non-array are still compatible
                const sigs = is_array_sig ? tx.sign : [tx.sign];
                const { sign, ...txWithoutSign } = tx;
                const authorized = (0, Utils_1.verifyMultiSigs)(txWithoutSign, sigs, multiSigPublicKeys, requiredSigs, security_1.DevSecurityLevel.HIGH);
                if (!authorized) {
                    Logger.mainLogger.info(`ChangeConfig or ChangeNetworkParam failed verification ${JSON.stringify(tx)}`);
                    return { result: 'fail', reason: 'Invalid Signature' };
                }
                return { result: 'pass', reason: 'valid' };
            }
            else if (tx.internalTXType === internalTxType_1.InternalTXType.SetCertTime) {
                return { result: 'pass', reason: 'valid' };
            }
            else if (tx.internalTXType === internalTxType_1.InternalTXType.InitRewardTimes) {
                const isValid = crypto.verifyObj(tx);
                if (!isValid) {
                    Logger.mainLogger.info(`Init reward tx failed verification ${JSON.stringify(tx)}`);
                    return { result: 'fail', reason: 'Invalid Signature' };
                }
                Logger.mainLogger.info(`Init reward tx passed ${JSON.stringify(tx)}`);
                return { result: 'pass', reason: 'valid' };
            }
            else if (tx.internalTXType === internalTxType_1.InternalTXType.TransferFromSecureAccount) {
                const verifyResult = validateTransferFromSecureAccount(tx);
                return { result: verifyResult.success ? 'pass' : 'fail', reason: verifyResult.reason };
            }
            else {
                const isValid = crypto.verifyObj(tx);
                if (!isValid) {
                    Logger.mainLogger.info(`Single signed tx failed ${JSON.stringify(tx)}`);
                    return { result: 'fail', reason: 'Invalid Signature' };
                }
                return { result: 'pass', reason: 'valid' };
            }
        }
        if (tx?.isDebugTx === true) {
            Logger.mainLogger.info(`Debug tx allowed ${JSON.stringify(tx)}`);
            return { result: 'pass', reason: 'all_allowed' };
        }
        // verify coin transfer tx
        try {
            const transaction = getTransactionObj(tx);
            const isSigned = transaction.isSigned();
            if (!isSigned) {
                return { result: 'fail', reason: 'invalid - signs missing' };
            }
            const { address: senderAddress, isValid: isSignatureValid } = getTxSenderAddress(transaction);
            if (!isSignatureValid) {
                return { result: 'fail', reason: 'invalid - signature' };
            }
            return { result: 'pass', reason: 'valid' };
        }
        catch (error) {
            return { result: 'fail', reason: 'invalid could not get sender address' };
        }
    }
    catch (err) {
        Logger.mainLogger.error(err);
    }
    return { result: 'fail', reason: `tx verification error :  ${tx}` };
};
exports.verifyTransaction = verifyTransaction;
function isInternalTXGlobal(internalTx) {
    return (internalTx.internalTXType === internalTxType_1.InternalTXType.SetGlobalCodeBytes ||
        internalTx.internalTXType === internalTxType_1.InternalTXType.ApplyChangeConfig ||
        internalTx.internalTXType === internalTxType_1.InternalTXType.InitNetwork ||
        internalTx.internalTXType === internalTxType_1.InternalTXType.ApplyNetworkParam);
}
exports.isInternalTXGlobal = isInternalTXGlobal;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function isInternalTx(timestampedTx) {
    if (timestampedTx && timestampedTx.raw)
        return false;
    if (timestampedTx && timestampedTx.isInternalTx)
        return true;
    if (timestampedTx && timestampedTx.tx && timestampedTx.tx.isInternalTx)
        return true;
    return false;
}
exports.isInternalTx = isInternalTx;
function validateTransferFromSecureAccount(tx) {
    try {
        if (tx.internalTXType !== internalTxType_1.InternalTXType.TransferFromSecureAccount) {
            return { success: false, reason: 'Invalid Secure Account Transaction type' };
        }
        if (typeof tx.amount !== 'string' || !/^\d+$/.test(tx.amount)) {
            return { success: false, reason: 'Invalid amount format' };
        }
        if (BigInt(tx.amount) <= 0) {
            return { success: false, reason: 'Amount is negative or zero' };
        }
        if (typeof tx.accountName !== 'string' || tx.accountName.trim() === '') {
            return { success: false, reason: 'Invalid account name' };
        }
        if (typeof tx.nonce !== 'number' || tx.nonce < 0) {
            return { success: false, reason: 'Invalid nonce' };
        }
        if (!tx.chainId || typeof tx.chainId !== 'string') {
            return { success: false, reason: 'Invalid chainId' };
        }
        const secureAccounts = getSecureAccounts();
        const secureAccountData = secureAccounts.get(tx.accountName);
        if (!secureAccountData) {
            return { success: false, reason: 'Secure Account not found' };
        }
        // verify signatures
        // check if tx.sign is not an array
        if (!Array.isArray(tx.sign)) {
            return { success: false, reason: 'tx.sign is not an array' };
        }
        if (tx.sign.length === 0) {
            return { success: false, reason: 'Missing signatures' };
        }
        const txData = {
            account: tx.amount,
            accountName: tx.accountName,
            nonce: tx.nonce,
            chainId: tx.chainId,
        };
        const allowedPublicKeys = (0, exports.getMultisigPublicKeys)();
        const requiredSigs = Math.max(3, minMultiSigRequiredForGlobalTxs);
        const isSignatureValid = (0, Utils_1.verifyMultiSigs)(txData, tx.sign, allowedPublicKeys, requiredSigs, security_1.DevSecurityLevel.HIGH);
        if (!isSignatureValid) {
            Logger.mainLogger.error(`Signature verification failed for secure account transfer transaction : ${lib_types_1.Utils.safeStringify(tx)}`);
            return { success: false, reason: 'Invalid signature' };
        }
        return { success: true, reason: 'valid' };
    }
    catch (error) {
        Logger.mainLogger.error(`error in validation of secure account transaction verification ${lib_types_1.Utils.safeStringify(tx)}`, ' and error is : ', error);
        return { success: false, reason: 'error occurred in verification of secure account transfer transaction' };
    }
}
exports.validateTransferFromSecureAccount = validateTransferFromSecureAccount;
function getTransactionObj(tx) {
    if (!tx.raw)
        throw Error('fail');
    let transactionObj;
    const serializedInput = Buffer.from(tx.raw);
    try {
        transactionObj = tx_1.TransactionFactory.fromSerializedData(serializedInput);
    }
    catch (e) {
        // do nothing here
    }
    if (!transactionObj) {
        try {
            transactionObj = tx_1.TransactionFactory.fromSerializedData(serializedInput);
        }
        catch (e) {
            Logger.mainLogger.error(`error in getTransactionObject ${tx}`);
        }
    }
    if (transactionObj) {
        Object.freeze(transactionObj);
        return transactionObj;
    }
    else
        throw Error('tx obj fail');
}
exports.getTransactionObj = getTransactionObj;
function getTxSenderAddress(tx) {
    try {
        const rawTx = '0x' + toHexString(tx.serialize());
        const { address, isValid, gasValid } = (0, lib_net_1.getSenderAddress)(rawTx);
        return { address: util_1.Address.fromString(address), isValid, gasValid };
    }
    catch (e) {
        Logger.mainLogger.error('Error getting sender address from tx', e);
    }
    return { address: null, isValid: false, gasValid: false };
}
exports.getTxSenderAddress = getTxSenderAddress;
function toHexString(byteArray) {
    return Array.from(byteArray, (byte) => {
        return ('0' + (byte & 0xff).toString(16)).slice(-2);
    }).join('');
}
//# sourceMappingURL=transactionVerification.js.map