"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.verifyAccountHash = exports.accountSpecificHash = exports.AccountType = void 0;
const crypto = require("../Crypto");
const Helpers_1 = require("../types/ajv/Helpers");
const AJVSchemaEnum_1 = require("../types/enum/AJVSchemaEnum");
const verifyGlobalTxReceipt_1 = require("./verifyGlobalTxReceipt");
// account types in Shardeum
var AccountType;
(function (AccountType) {
    AccountType[AccountType["Account"] = 0] = "Account";
    AccountType[AccountType["ContractStorage"] = 1] = "ContractStorage";
    AccountType[AccountType["ContractCode"] = 2] = "ContractCode";
    AccountType[AccountType["Receipt"] = 3] = "Receipt";
    AccountType[AccountType["Debug"] = 4] = "Debug";
    AccountType[AccountType["NetworkAccount"] = 5] = "NetworkAccount";
    AccountType[AccountType["NodeAccount"] = 6] = "NodeAccount";
    AccountType[AccountType["NodeRewardReceipt"] = 7] = "NodeRewardReceipt";
    AccountType[AccountType["DevAccount"] = 8] = "DevAccount";
    AccountType[AccountType["NodeAccount2"] = 9] = "NodeAccount2";
    AccountType[AccountType["StakeReceipt"] = 10] = "StakeReceipt";
    AccountType[AccountType["UnstakeReceipt"] = 11] = "UnstakeReceipt";
    AccountType[AccountType["InternalTxReceipt"] = 12] = "InternalTxReceipt";
    AccountType[AccountType["SecureAccount"] = 13] = "SecureAccount";
})(AccountType = exports.AccountType || (exports.AccountType = {}));
const accountSpecificHash = (account) => {
    let hash;
    delete account.hash;
    if (account.accountType === AccountType.NetworkAccount ||
        account.accountType === AccountType.NodeAccount ||
        account.accountType === AccountType.NodeAccount2 ||
        account.accountType === AccountType.NodeRewardReceipt ||
        account.accountType === AccountType.StakeReceipt ||
        account.accountType === AccountType.UnstakeReceipt ||
        account.accountType === AccountType.InternalTxReceipt ||
        account.accountType === AccountType.DevAccount ||
        account.accountType === AccountType.SecureAccount) {
        account.hash = crypto.hashObj(account);
        return account.hash;
    }
    if (account.accountType === AccountType.Account) {
        const { account: EVMAccountInfo, operatorAccountInfo, timestamp } = account;
        const accountData = operatorAccountInfo
            ? { EVMAccountInfo, operatorAccountInfo, timestamp }
            : { EVMAccountInfo, timestamp };
        hash = crypto.hashObj(accountData);
    }
    else if (account.accountType === AccountType.Debug) {
        hash = crypto.hashObj(account);
    }
    else if (account.accountType === AccountType.ContractStorage) {
        hash = crypto.hashObj({ key: account.key, value: account.value });
    }
    else if (account.accountType === AccountType.ContractCode) {
        hash = crypto.hashObj({ key: account.codeHash, value: account.codeByte });
    }
    else if (account.accountType === AccountType.Receipt) {
        hash = crypto.hashObj({ key: account.txId, value: account.receipt });
    }
    // hash = hash + '0'.repeat(64 - hash.length)
    account.hash = hash;
    return hash;
};
exports.accountSpecificHash = accountSpecificHash;
const verifyAccountHash = (receipt, failedReasons = [], nestedCounterMessages = []) => {
    try {
        let globalReceiptValidationErrors; // This is used to store the validation errors of the globalTxReceipt
        try {
            globalReceiptValidationErrors = (0, Helpers_1.verifyPayload)(AJVSchemaEnum_1.AJVSchemaEnum.GlobalTxReceipt, receipt?.signedReceipt);
        }
        catch (error) {
            globalReceiptValidationErrors = true;
            failedReasons.push(`Invalid Global Tx Receipt error: ${error}. txId ${receipt.tx.txId} , cycle ${receipt.cycle} , timestamp ${receipt.tx.timestamp}`);
            nestedCounterMessages.push(`Invalid Global Tx Receipt error: ${error}. txId ${receipt.tx.txId} , cycle ${receipt.cycle} , timestamp ${receipt.tx.timestamp}`);
            return false;
        }
        if (!globalReceiptValidationErrors) {
            const result = (0, verifyGlobalTxReceipt_1.verifyGlobalTxAccountChange)(receipt, failedReasons, nestedCounterMessages);
            if (!result)
                return false;
            return true;
        }
        const signedReceipt = receipt.signedReceipt;
        const { accountIDs, afterStateHashes, beforeStateHashes } = signedReceipt.proposal;
        if (accountIDs.length !== afterStateHashes.length) {
            failedReasons.push(`Modified account count specified in the receipt and the actual updated account count does not match! ${receipt.tx.txId} , ${receipt.cycle} , ${receipt.tx.timestamp}`);
            nestedCounterMessages.push(`Modified account count specified in the receipt and the actual updated account count does not match!`);
            return false;
        }
        if (beforeStateHashes.length !== afterStateHashes.length) {
            failedReasons.push(`Account state hash before and after count does not match! ${receipt.tx.txId} , ${receipt.cycle} , ${receipt.tx.timestamp}`);
            nestedCounterMessages.push(`Account state hash before and after count does not match!`);
            return false;
        }
        for (const [index, accountId] of accountIDs.entries()) {
            const accountData = receipt.afterStates.find((acc) => acc.accountId === accountId);
            if (accountData === undefined) {
                failedReasons.push(`Account not found in the receipt's afterStates | Acc-ID: ${accountId}, txId: ${receipt.tx.txId}, Cycle: ${receipt.cycle}, timestamp: ${receipt.tx.timestamp}`);
                nestedCounterMessages.push(`Account not found in the receipt`);
                return false;
            }
            const calculatedAccountHash = (0, exports.accountSpecificHash)(accountData.data);
            // eslint-disable-next-line security/detect-object-injection
            const expectedAccountHash = afterStateHashes[index];
            if (calculatedAccountHash !== expectedAccountHash) {
                failedReasons.push(`Account hash does not match | Acc-ID: ${accountId}, txId: ${receipt.tx.txId}, Cycle: ${receipt.cycle}, timestamp: ${receipt.tx.timestamp}`);
                nestedCounterMessages.push(`Account hash does not match`);
                return false;
            }
        }
        return true;
    }
    catch (e) {
        console.error(`Error in verifyAccountHash`, e);
        failedReasons.push(`Error in verifyAccountHash ${e}`);
        nestedCounterMessages.push('Error in verifyAccountHash');
        return false;
    }
};
exports.verifyAccountHash = verifyAccountHash;
//# sourceMappingURL=calculateAccountHash.js.map