"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.customAxios = exports.customFetch = void 0;
const axios_1 = require("axios");
const Config_1 = require("../Config");
const node_fetch_1 = require("node-fetch");
const lib_types_1 = require("@shardeum-foundation/lib-types");
const stream_1 = require("stream");
/**
 * A custom fetch function with a max response body size limit.
 * @param input URL or Request object.
 * @param init Optional fetch configuration.
 * @param maxBytes Maximum response size in bytes (default: config.maxResponseSize).
 * @returns A Response.
 */
async function customFetch(input, init, maxBytes) {
    const downloadLimit = maxBytes ?? Config_1.config.maxResponseSize;
    return await (0, node_fetch_1.default)(input, { ...init, size: downloadLimit });
}
exports.customFetch = customFetch;
/**
 * Creates a custom axios instance with size limiting
 * @param maxBytes Maximum response size in bytes (default: 10MB)
 * @param axiosConfig Additional axios axiosConfig options
 * @returns Custom axios instance
 */
function customAxios(maxBytes, axiosConfig = {}) {
    const downloadLimit = maxBytes ?? Config_1.config.maxResponseSize;
    const userRequestedType = axiosConfig.responseType ?? 'json';
    axiosConfig.responseType = 'stream';
    const instance = axios_1.default.create({
        ...axiosConfig,
        validateStatus: () => true,
    });
    instance.interceptors.request.use((request) => {
        const source = axios_1.default.CancelToken.source();
        request.cancelToken = source.token;
        request._cancelSource = source;
        return request;
    });
    instance.interceptors.response.use(async (response) => {
        const contentLength = parseInt(response.headers['content-length'] || '0', 10);
        if (contentLength > 0 && contentLength > downloadLimit) {
            ;
            response.config._cancelSource?.cancel(`Response content-length ${contentLength} exceeds limit of ${downloadLimit}`);
            throw new Error(`Response size exceeds limit of ${downloadLimit} bytes`);
        }
        const stream = response.data;
        let totalBytes = 0;
        const chunks = [];
        return new Promise((resolve, reject) => {
            stream.on('data', (chunk) => {
                totalBytes += chunk.length;
                if (totalBytes > downloadLimit) {
                    stream.destroy(new Error(`Response size exceeds limit of ${downloadLimit} bytes`));
                    return;
                }
                chunks.push(chunk);
            });
            stream.on('end', () => {
                const fullBuffer = Buffer.concat(chunks);
                switch (userRequestedType) {
                    case 'stream': {
                        // If user truly wants a stream, we can either:
                        const pass = new stream_1.PassThrough();
                        pass.end(fullBuffer);
                        response.data = pass;
                        break;
                    }
                    case 'arraybuffer':
                        response.data = fullBuffer;
                        break;
                    case 'json':
                    default:
                        try {
                            response.data = lib_types_1.Utils.safeJsonParse(fullBuffer.toString('utf8'));
                        }
                        catch (err) {
                            return reject(new Error(`Failed to parse JSON (size: ${fullBuffer.length} bytes): ${err.message}`));
                        }
                        break;
                }
                resolve(response);
            });
            stream.on('error', (err) => {
                reject(err);
            });
        });
    }, (error) => {
        // Distinguish cancellation due to size from other errors
        if (axios_1.default.isCancel(error)) {
            throw new Error(`Response size exceeds limit of ${downloadLimit} bytes`);
        }
        throw error;
    });
    return instance;
}
exports.customAxios = customAxios;
//# sourceMappingURL=customHttpFunctions.js.map